/*Copyright (C) 2010-2011, Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include "silkpriv.h"
#include "celtpriv.h"
#include "opuspriv.h"



opus_stream_decoder *opus_stream_decoder_alloc(int _nchannels,
 opus_read_func _read,void *_stream){
  opus_stream_decoder *dec;
  silk_decoder        *silkdec;
  celt_decoder        *celtdec;
  dec=(opus_stream_decoder *)calloc(1,sizeof(*dec));
  if(dec!=NULL){
    silkdec=silk_decoder_alloc(_nchannels);
    if(silkdec!=NULL){
      celtdec=celt_decoder_alloc(&CELT_MODE_48000_960_120,_nchannels);
      if(celtdec!=NULL){
        dec->silkdec=silkdec;
        dec->celtdec=celtdec;
        dec->read=_read;
        dec->stream=_stream;
        dec->nframes=-1;
        return dec;
      }
      silk_decoder_free(silkdec);
    }
    opus_stream_decoder_free(dec);
  }
  return NULL;
}

void opus_stream_decoder_free(opus_stream_decoder *_dec){
  if(_dec!=NULL){
    silk_decoder_free(_dec->silkdec);
    celt_decoder_free(_dec->celtdec);
    free(_dec);
  }
}


static int opus_read_length(opus_stream_decoder *_dec,
 opus_uint32 *_packet_length){
  unsigned char len[2];
  if(*_packet_length<1||(*_dec->read)(len,1,1,_dec->stream)<1){
    return -EINVAL;
  }
  (*_packet_length)--;
  if(len[0]>=252){
    if(*_packet_length<1||(*_dec->read)(len+1,1,1,_dec->stream)<1){
      return -EINVAL;
    }
    (*_packet_length)--;
    return len[1]*4+len[0];
  }
  return len[0];
}

int opus_decode_frame(opus_stream_decoder *_dec,float *_pcm,
 opus_uint32 _packet_length){
  static const unsigned char OPUS_B_END[4]={13,17,19,21};
  opus_range_coder rc;
  unsigned char    tmp[4];
  unsigned char    frame[1275];
  int              nchannels;
  int              config;
  int              nframes;
  int              frame_length;
  int              ret;
  nframes=_dec->nframes;
  if(nframes<0){
    static unsigned char SILK_NFRAMES_MAX[4]={12,6,3,2};
    int self_delimited;
    int nframes_max;
    int code;
    self_delimited=_packet_length==0;
    if(self_delimited)_packet_length=0xFFFFFFFFU;
    if(_packet_length<1||(*_dec->read)(tmp,1,1,_dec->stream)<1)return -EINVAL;
    _packet_length--;
    _dec->toc=tmp[0];
    code=tmp[0]&3;
    nchannels=(tmp[0]>>2&1)+1;
    config=tmp[0]>>3;
    if(config<12)nframes_max=SILK_NFRAMES_MAX[config&3];
    else if(config<16)return -EDOM;
    else nframes_max=48>>(config&3);
    _dec->padding_length=0;
    switch(code){
      case 0:{
        if(self_delimited){
          frame_length=opus_read_length(_dec,&_packet_length);
          if(frame_length<0)return frame_length;
        }
        else{
          if(_packet_length>1275)return -EINVAL;
          frame_length=(int)_packet_length;
        }
        _dec->frame_lengths[0]=(opus_int16)frame_length;
        nframes=1;
      }break;
      case 1:{
        if(self_delimited){
          frame_length=opus_read_length(_dec,&_packet_length);
          if(frame_length<0)return frame_length;
        }
        else{
          if(_packet_length>1275*2||_packet_length&1)return -EINVAL;
          frame_length=(int)(_packet_length>>1);
        }
        _dec->frame_lengths[1]=(opus_int16)frame_length;
        _dec->frame_lengths[0]=(opus_int16)frame_length;
        nframes=2;
      }break;
      case 2:{
        frame_length=opus_read_length(_dec,&_packet_length);
        if(frame_length<0)return frame_length;
        if((opus_uint32)frame_length>_packet_length)return -EINVAL;
        _packet_length-=frame_length;
        _dec->frame_lengths[1]=(opus_int16)frame_length;
        if(self_delimited){
          frame_length=opus_read_length(_dec,&_packet_length);
          if(frame_length<0)return frame_length;
        }
        else{
          if(_packet_length>1275)return -EINVAL;
          frame_length=(int)_packet_length;
        }
        _dec->frame_lengths[0]=(opus_int16)frame_length;
        nframes=2;
      }break;
      default:{
        int framei;
        int padded;
        int vbr;
        if(_packet_length<1||(*_dec->read)(tmp+1,1,1,_dec->stream)<1){
          return -EINVAL;
        }
        _packet_length--;
        nframes=tmp[1]&0x3F;
        if(nframes<1||nframes>nframes_max)return -EDOM;
        padded=tmp[1]&0x40;
        vbr=tmp[1]&0x80;
        if(padded){
          opus_uint32 padding_length;
          padding_length=0;
          for(;;){
            if(_packet_length<1||(*_dec->read)(tmp+2,1,1,_dec->stream)<1){
              return -EINVAL;
            }
            _packet_length--;
            if(tmp[2]<255)break;
            if(_packet_length<255)return -EINVAL;
            _packet_length-=255;
            padding_length+=255;
          }
          if(_packet_length<tmp[2])return -EINVAL;
          _packet_length-=tmp[2];
          padding_length+=tmp[2];
          _dec->padding_length=padding_length;
        }
        if(vbr){
          for(framei=0;framei<nframes+self_delimited-1;framei++){
            frame_length=opus_read_length(_dec,&_packet_length);
            if(frame_length<0)return frame_length;
            if((opus_uint32)frame_length>_packet_length)return -EINVAL;
            _packet_length-=frame_length;
            _dec->frame_lengths[nframes-framei-1]=(opus_int16)frame_length;
          }
          if(!self_delimited){
            if(_packet_length>1275)return -EINVAL;
            _dec->frame_lengths[0]=(opus_int16)_packet_length;
          }
        }
        else{
          if(self_delimited){
            frame_length=opus_read_length(_dec,&_packet_length);
            if(frame_length<0)return frame_length;
          }
          else{
            if(_packet_length>1275U*nframes)return -EINVAL;
            frame_length=_packet_length/nframes;
            if((opus_uint32)frame_length*nframes!=_packet_length)return -EINVAL;
          }
          for(framei=0;framei<nframes;framei++){
            _dec->frame_lengths[framei]=(opus_int16)frame_length;
          }
        }
      }break;
    }
  }
  else{
    nchannels=(_dec->toc>>2&1)+1;
    config=_dec->toc>>3;
  }
  _dec->nframes=--nframes;
  if(nframes<0)return 0;
  frame_length=_dec->frame_lengths[nframes];
  if((*_dec->read)(frame,frame_length,1,_dec->stream)<1)return -EINVAL;
  orc_dec_init(&rc,frame,frame_length);
  if(config<12){
    int bw;
    int fsi;
    bw=config>>2;
    fsi=config&3;
    ret=silk_decode_impl(_dec->silkdec,&rc,_pcm,nchannels,bw,fsi);
    /*Check for redundancy.*/
    if(orc_tell(&rc)+17<=(opus_uint32)frame_length*8)return -EDOM;
  }
  else if(config<16)return -EDOM;
  else{
    int b_end;
    int lm;
    b_end=OPUS_B_END[config-16>>2];
    lm=config&3;
    ret=celt_decode_impl(_dec->celtdec,&rc,_pcm,nchannels,0,b_end,lm);
  }
  if(nframes==0){
    opus_uint32 padding_length;
    padding_length=_dec->padding_length;
    while(padding_length>1275){
      if((*_dec->read)(frame,1275,1,_dec->stream)<1)return -EINVAL;
      padding_length-=1275;
    }
    if(padding_length>0&&(*_dec->read)(frame,padding_length,1,_dec->stream)<1){
      return -EINVAL;
    }
  }
  _dec->rng=rc.rng;
  return ret;
}



#if 1
#include <stdio.h>

int main(int _argc,const char **_argv){
  opus_stream_decoder *dec;
  const celt_mode     *mode;
  FILE                *fin;
  FILE                *fout;
  float               *pcm;
  opus_int16          *pcm16;
  int                  frame_size_max;
  int                  nchannels;
  int                  skip;
  int                  ret;
  if(_argc!=4){
    fprintf(stderr,
     "Usage: %s <channels> <input.raw> <output.sw>\n",
     _argv[0]);
    return EXIT_FAILURE;
  }
  nchannels=atoi(_argv[1]);
  fin=fopen(_argv[2],"rb");
  fout=fin!=NULL?fopen(_argv[3],"wb"):NULL;
  dec=fin!=NULL?
   opus_stream_decoder_alloc(nchannels,(opus_read_func)fread,fin):NULL;
  if(fin==NULL||fout==NULL||dec==NULL){
    fprintf(stderr,"EINVAL: %i %s %s\n",nchannels,_argv[2],_argv[3]);
    return EXIT_FAILURE;
  }
  mode=dec->celtdec->mode;
  frame_size_max=mode->mdct.short_size<<mode->lm_max;
  pcm=(float *)malloc(frame_size_max*nchannels*sizeof(*pcm));
  pcm16=(opus_int16 *)malloc(frame_size_max*nchannels*sizeof(*pcm16));
  skip=mode->mdct.overlap;
  ret=EXIT_SUCCESS;
  for(;;){
    unsigned char tmp[4];
    opus_uint32   packet_length;
    opus_uint32   rng;
    if(fread(tmp,4*sizeof(*tmp),1,fin)<1)break;
    packet_length=
     (opus_uint32)tmp[0]<<24|(opus_uint32)tmp[1]<<16|tmp[2]<<8|tmp[3];
    if(fread(tmp,4,1,fin)<1){
      fprintf(stderr,"Invalid packet.\n");
      ret=EXIT_FAILURE;
      break;
    }
    rng=(opus_uint32)tmp[0]<<24|(opus_uint32)tmp[1]<<16|tmp[2]<<8|tmp[3];
    for(;;){
      int frame_size;
      int i;
      frame_size=opus_decode_frame(dec,pcm,packet_length);
      if(frame_size==0)break;
      if(frame_size<0){
        fprintf(stderr,"Invalid packet.\n");
        ret=EXIT_FAILURE;
        break;
      }
      {
        static int nsamples=0;
        static int framei=0;
        printf("frame %4i: %3i %i\n",framei,frame_size,nsamples);
        framei++;
        nsamples+=frame_size;
      }
      for(i=0;i<frame_size*nchannels;i++){
        pcm16[i]=(opus_int16)OPUS_CLAMP(-32768,floor(32768*pcm[i]+0.5F),32767);
      }
      if(frame_size>skip&&
       fwrite(pcm16+nchannels*skip,(frame_size-skip)*nchannels*2,1,fout)<1){
        ret=EXIT_FAILURE;
        break;
      }
      skip=0;
    }
    if(ret!=EXIT_SUCCESS)break;
    if(rng!=0&&rng!=dec->rng){
      fprintf(stderr,"RNG mismatch: 0x%08lX!=0x%08lX\n",
       (long)rng,(long)dec->rng);
      ret=EXIT_FAILURE;
      break;
    }
  }
  free(pcm16);
  free(pcm);
  opus_stream_decoder_free(dec);
  fclose(fout);
  fclose(fin);
  return ret;
}
#endif
