/*Copyright (C) 2010-2011, Mozilla Corporation.

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.*/
#include <stdlib.h>
#include <alloca.h>
#include <errno.h>
#include <string.h>
#include "silkpriv.h"



typedef struct silk_ltp_params      silk_ltp_params;
typedef struct silk_cond_coding_ctx silk_cond_coding_ctx;



#define SILK_CODE_INDEPENDENTLY_NO_LTP_SCALING (-1)
#define SILK_CODE_CONDITIONALLY (0)
#define SILK_CODE_INDEPENDENTLY (1)

#define SILK_TYPE_INACTIVE (0)
#define SILK_TYPE_UNVOICED (1)
#define SILK_TYPE_VOICED   (2)



/*OPT: Most CPUs have a single instruction that does this.*/
static int silk_ilog(opus_uint32 _v){
  int ret;
  for(ret=0;_v;ret++)_v>>=1;
  return ret;
}

silk_decoder *silk_decoder_alloc(int _nchannels){
  silk_decoder *dec;
  if(_nchannels<1||_nchannels>2)return NULL;
  dec=(silk_decoder *)calloc(1,sizeof(*dec)
   +(_nchannels-1)*sizeof(dec->chan[0]));
  if(dec==NULL)return NULL;
  dec->stereo=_nchannels-1;
  dec->prev_bw=-1;
  dec->chan[0].prev_gain_q16=65536;
  dec->chan[_nchannels-1].prev_gain_q16=65536;
  return dec;
}

void silk_decoder_free(silk_decoder *_dec){
  if(_dec!=NULL){
    free(_dec);
  }
}


static void silk_decode_header_bits(opus_range_coder *_rc,
 int _vad[3][2],int _lbrr[3][2],int _stereo,int _nframes){
  int ci;
  int fi;
  for(ci=0;ci<=_stereo;ci++){
    for(fi=0;fi<_nframes;fi++)_vad[fi][ci]=orc_dec_bit_logp(_rc,1);
    _lbrr[0][ci]=orc_dec_bit_logp(_rc,1);
  }
  if(_nframes>1){
    for(ci=0;ci<=_stereo;ci++){
      if(_lbrr[0][ci]){
        int lbrr_flags;
        lbrr_flags=orc_dec_pdf(_rc,SILK_LBRR_FLAG_PDFS+(_nframes-2)*4,8);
        for(fi=0;fi<_nframes;fi++)_lbrr[fi][ci]=lbrr_flags>>fi&1;
      }
    }
  }
}

static void silk_decode_stereo_weights(opus_range_coder *_rc,
 opus_int16 _w_q13[2]){
  int n;
  int i0;
  int i1;
  int i2;
  int i3;
  int sw0_q13;
  int sw1_q13;
  int w1_q13;
  n=orc_dec_pdf(_rc,SILK_STEREO_WEIGHTS_STAGE1_PDF,8);
  i0=orc_dec_pdf(_rc,SILK_STEREO_WEIGHTS_STAGE2_PDF,8);
  i1=2*orc_dec_pdf(_rc,SILK_STEREO_WEIGHTS_STAGE3_PDF,8)+1;
  i2=orc_dec_pdf(_rc,SILK_STEREO_WEIGHTS_STAGE2_PDF,8);
  i3=2*orc_dec_pdf(_rc,SILK_STEREO_WEIGHTS_STAGE3_PDF,8)+1;
  if(_w_q13!=NULL){
    i0+=3*(n/5);
    i2+=3*(n%5);
    sw0_q13=SILK_STEREO_WEIGHTS_Q13[i2];
    sw1_q13=SILK_STEREO_WEIGHTS_Q13[i2+1];
    w1_q13=sw0_q13+(int)((sw1_q13-sw0_q13)*(opus_int32)6554>>16)*i3;
    sw0_q13=SILK_STEREO_WEIGHTS_Q13[i0];
    sw1_q13=SILK_STEREO_WEIGHTS_Q13[i0+1];
    _w_q13[0]=(opus_int16)(sw0_q13
     +(int)((sw1_q13-sw0_q13)*(opus_int32)6554>>16)*i1-w1_q13);
    _w_q13[1]=(opus_int16)w1_q13;
  }
}



struct silk_cond_coding_ctx{
  opus_int16 previous_lag;
};



static opus_int32 silk_gain_log2lin(int _log_gain){
  opus_int32 ef_q7;
  int        log_gain_q7;
  int        ipart;
  int        fpart;
  log_gain_q7=(0x1D1C71*_log_gain>>16)+2090;
  ipart=log_gain_q7>>7;
  fpart=log_gain_q7&127;
  ef_q7=(-174*fpart*(opus_int32)(128-fpart)>>16)+fpart;
  return (1<<ipart)+(ipart<16?ef_q7>>7<<ipart:ef_q7<<ipart-7);
}

static int silk_decode_gains(opus_range_coder *_rc,opus_int32 _gain_q16[4],
 int _log_gain,int _signal_type,int _coding_type,int _nsubframes){
  int gain_index;
  int sfi;
  if(_coding_type==SILK_CODE_CONDITIONALLY)sfi=0;
  else{
    gain_index=orc_dec_pdf(_rc,SILK_GAIN_HIGH_PDFS+8*_signal_type,8)<<3|
     orc_dec_pdf(_rc,SILK_GAIN_LOW_PDF,8);
    _log_gain=OPUS_MAX(_log_gain-16,gain_index);
    _gain_q16[0]=silk_gain_log2lin(_log_gain);
    sfi=1;
  }
  for(;sfi<_nsubframes;sfi++){
    gain_index=orc_dec_pdf(_rc,SILK_GAIN_DELTA_PDF,8);
    _log_gain=OPUS_CLAMP(0,OPUS_MAX(2*gain_index-16,_log_gain+gain_index-4),63);
    _gain_q16[sfi]=silk_gain_log2lin(_log_gain);
  }
  return _log_gain;
}

static int opus_int16_compare(const void *_a,const void *_b){
  int a;
  int b;
  a=*(const opus_int16 *)_a;
  b=*(const opus_int16 *)_b;
  return (a>b)-(a<b);
}

static void silk_stabilize_nlsfs(const silk_nlsf_codebook *_cb,
 opus_int16 _nlsf_q15[]){
  const opus_int16 *min_spacing_q15;
  int               pn_q15;
  int               n_q15;
  int               order;
  int               fi;
  int               k;
  order=_cb->order;
  min_spacing_q15=_cb->min_spacing_q15;
  for(k=0;k<20;k++){
    int worst_fi;
    int worst_delta_q15;
    worst_fi=order;
    worst_delta_q15=32768-_nlsf_q15[order-1]-min_spacing_q15[order];
    pn_q15=0;
    for(fi=0;fi<order;fi++){
      int delta_q15;
      n_q15=_nlsf_q15[fi];
      delta_q15=n_q15-pn_q15-min_spacing_q15[fi];
      pn_q15=n_q15;
      if(delta_q15<worst_delta_q15){
        worst_delta_q15=delta_q15;
        worst_fi=fi;
      }
    }
    if(worst_delta_q15>=0)return;
    if(worst_fi==0)_nlsf_q15[0]=min_spacing_q15[0];
    else if(worst_fi==order)_nlsf_q15[order-1]=32768-min_spacing_q15[order];
    else{
      int min_center_q15;
      int max_center_q15;
      int center_freq_q15;
      min_center_q15=min_spacing_q15[worst_fi]>>1;
      max_center_q15=32768-(min_spacing_q15[worst_fi]>>1);
      for(fi=0;fi<worst_fi;fi++)min_center_q15+=min_spacing_q15[fi];
      for(fi=worst_fi+1;fi<=order;fi++)max_center_q15-=min_spacing_q15[fi];
      center_freq_q15=OPUS_CLAMP(min_center_q15,
       _nlsf_q15[worst_fi-1]+_nlsf_q15[worst_fi]+1>>1,max_center_q15);
      pn_q15=center_freq_q15-(min_spacing_q15[worst_fi]>>1);
      n_q15=pn_q15+min_spacing_q15[worst_fi];
      _nlsf_q15[worst_fi-1]=(opus_int16)pn_q15;
      _nlsf_q15[worst_fi]=(opus_int16)n_q15;
    }
  }
  qsort(_nlsf_q15,order,sizeof(*_nlsf_q15),opus_int16_compare);
  n_q15=0;
  for(fi=0;fi<order;fi++){
    n_q15=OPUS_MAX(_nlsf_q15[fi],n_q15+min_spacing_q15[fi]);
    _nlsf_q15[fi]=(opus_int16)n_q15;
  }
  n_q15=32768;
  for(fi=order;fi-->0;){
    n_q15=OPUS_MIN(_nlsf_q15[fi],n_q15-min_spacing_q15[fi+1]);
    _nlsf_q15[fi]=(opus_int16)n_q15;
  }
}

static void silk_decode_nlsfs(const silk_nlsf_codebook *_cb,
 opus_range_coder *_rc,opus_int16 _nlsf_q15[],int _signal_type){
  const unsigned char *stage2_indices;
  unsigned char        stage2_pdfi[16];
  unsigned char        stage2_wi[16];
  int                  i2[16];
  int                  order;
  int                  i1;
  int                  fi;
  order=_cb->order;
  i1=orc_dec_pdf(_rc,_cb->stage1_pdfs+32*(_signal_type>>1),8);
  stage2_indices=_cb->stage2_indices+i1*(order>>1);
  for(fi=0;fi<order>>1;fi++){
    stage2_pdfi[2*fi]=stage2_indices[fi]&7;
    stage2_wi[2*fi]=stage2_indices[fi]>>3&1;
    stage2_pdfi[2*fi+1]=stage2_indices[fi]>>4&7;
    stage2_wi[2*fi+1]=stage2_indices[fi]>>7&1;
  }
  for(fi=0;fi<order;fi++){
    i2[fi]=orc_dec_pdf(_rc,_cb->stage2_pdfs+9*stage2_pdfi[fi],8)-4;
    if(i2[fi]==-4)i2[fi]-=orc_dec_pdf(_rc,SILK_NLSF_STAGE2_EXT_PDF,8);
    else if(i2[fi]==4)i2[fi]+=orc_dec_pdf(_rc,SILK_NLSF_STAGE2_EXT_PDF,8);
  }
  if(_nlsf_q15!=NULL){
    const unsigned char *stage1_cb_q8;
    const unsigned char *stage2_pred_weights_q8;
    opus_int32           qstep_q16;
    int                  pred_q10;
    int                  next_cb1_q8;
    int                  cb1_q8;
    stage1_cb_q8=_cb->stage1_cb_q8+i1*order;
    stage2_pred_weights_q8=_cb->stage2_pred_weights_q8;
    qstep_q16=_cb->stage2_quant_step_q16;
    pred_q10=0;
    next_cb1_q8=256;
    cb1_q8=stage1_cb_q8[order-1];
    for(fi=order-1;;fi--){
      opus_int32 res_q10;
      opus_int32 w2_q2;
      opus_int32 y;
      opus_int16 w_q9;
      opus_int32 w_q8;
      int        prev_cb1_q8;
      int        ipart;
      int        fpart;
      res_q10=pred_q10+(((i2[fi]<<10)-OPUS_SIGN(i2[fi])*102)*qstep_q16>>16);
      prev_cb1_q8=fi>0?stage1_cb_q8[fi-1]:0;
      w2_q2=1024/(cb1_q8-prev_cb1_q8)+1024/(next_cb1_q8-cb1_q8);
      ipart=silk_ilog(w2_q2);
      fpart=(w2_q2<<16)>>ipart+8&127;
      y=(ipart&1?32768:46214)>>(16-ipart>>1);
      w_q9=(opus_int16)(y+(213*fpart*y>>16));
      _nlsf_q15[fi]=
       (opus_int16)OPUS_CLAMP(0,(cb1_q8<<7)+(res_q10<<14)/w_q9,32767);
      if(fi<=0)break;
      w_q8=stage2_pred_weights_q8[fi-1+(order-1)*stage2_wi[fi-1]];
      pred_q10=(int)(res_q10*w_q8>>8);
      next_cb1_q8=cb1_q8;
      cb1_q8=prev_cb1_q8;
    }
    silk_stabilize_nlsfs(_cb,_nlsf_q15);
  }
}

static void silk_nlsf_poly(opus_int32 _p_q16[],opus_int32 _c_q17[],int _d_2){
  int ci;
  _p_q16[0]=65536;
  _p_q16[1]=-_c_q17[0];
  for(ci=1;ci<_d_2;ci++){
    int cj;
    _p_q16[ci+1]=2*_p_q16[ci-1]
     -(opus_int32)(_c_q17[2*ci]*(opus_int64)_p_q16[ci]+32768>>16);
    for(cj=ci;cj>=2;cj--){
      _p_q16[cj]+=_p_q16[cj-2]
       -(opus_int32)(_c_q17[2*ci]*(opus_int64)_p_q16[cj-1]+32768>>16);
    }
    _p_q16[1]-=_c_q17[2*ci];
  }
}

static void silk_bw_exp(opus_int32 _a32_q17[],int _order,opus_int32 _sc_q16){
  opus_uint32 uc_q16;
  int         ci;
  uc_q16=_sc_q16;
  for(ci=0;ci<_order;ci++){
    _a32_q17[ci]=(opus_int32)(_a32_q17[ci]*(opus_int64)_sc_q16>>16);
    _sc_q16=_sc_q16*uc_q16+32768>>16;
  }
}

static void silk_limit_lpc_magnitude(opus_int32 _a32_q17[],int _order){
  int k;
  int ci;
  for(k=0;k<10;k++){
    opus_int32 maxabs_q17;
    opus_int32 maxabs_q12;
    int        worst_ci;
    maxabs_q17=OPUS_ABS(_a32_q17[0]);
    worst_ci=0;
    for(ci=1;ci<_order;ci++){
      opus_int32 abs_q17;
      abs_q17=OPUS_ABS(_a32_q17[ci]);
      if(abs_q17>maxabs_q17){
        maxabs_q17=abs_q17;
        worst_ci=ci;
      }
    }
    maxabs_q12=OPUS_MIN(maxabs_q17+16>>5,16838);
    if(maxabs_q12<=32767)break;
    silk_bw_exp(_a32_q17,_order,
     65470-(maxabs_q12-32767<<14)/(maxabs_q12*(worst_ci+1)>>2));
  }
  if(k>=10){
    for(ci=0;ci<_order;ci++){
      _a32_q17[ci]=OPUS_CLAMP(-32768,_a32_q17[ci]+16>>5,32767)<<5;
    }
  }
}

static void silk_limit_lpc_gain(opus_int32 _a32_q17[],int _order){
  int k;
  int ci;
  for(k=0;k<16;k++){
    opus_int32 a32_q24[16];
    opus_int32 dc_resp;
    dc_resp=0;
    for(ci=0;ci<_order;ci++){
      opus_int32 a32_q12;
      a32_q12=_a32_q17[ci]+16>>5;
      dc_resp+=a32_q12;
      a32_q24[ci]=a32_q12<<12;
    }
    if(dc_resp<4096){
      opus_int32 inv_gain_q30;
      inv_gain_q30=0x40000000;
      for(ci=_order-1;;ci--){
        opus_int32 rc_q31;
        opus_int32 div_q30;
        opus_int32 gain_qb1;
        opus_int32 round_q24pb1;
        int        inv_qb2;
        int        err_q29;
        int        b1;
        int        b2;
        int        cj;
        if(OPUS_ABS(a32_q24[ci])>16773022)break;
        rc_q31=-a32_q24[ci]<<7;
        div_q30=0x40000000-(opus_int32)(rc_q31*(opus_int64)rc_q31>>32);
        inv_gain_q30=(opus_int32)(inv_gain_q30*(opus_int64)div_q30>>32)<<2;
        if(inv_gain_q30<107374)break;
        if(ci==0)return;
        b1=silk_ilog(div_q30);
        b2=b1-16;
        inv_qb2=(int)(0x1FFFFFFF/(int)(div_q30>>b2+1));
        err_q29=(int)(0x20000000
         -(opus_int32)(inv_qb2*(opus_int64)(div_q30<<15-b2)>>16));
        gain_qb1=((opus_int32)inv_qb2<<16)+(err_q29*(opus_int32)inv_qb2>>13);
        round_q24pb1=(opus_int32)1<<b1-1;
        for(cj=0;cj<ci+1>>1;cj++){
          opus_int32 num1_q24;
          opus_int32 num2_q24;
          num1_q24=a32_q24[cj]
           -(opus_int32)(a32_q24[ci-cj-1]*(opus_int64)rc_q31+0x40000000>>31);
          num2_q24=a32_q24[ci-cj-1]
           -(opus_int32)(a32_q24[cj]*(opus_int64)rc_q31+0x40000000>>31);
          a32_q24[cj]=
           (opus_int32)(num1_q24*(opus_int64)gain_qb1+round_q24pb1>>b1);
          a32_q24[ci-cj-1]=
           (opus_int32)(num2_q24*(opus_int64)gain_qb1+round_q24pb1>>b1);
        }
      }
    }
    silk_bw_exp(_a32_q17,_order,65536-((opus_int32)1<<k+1));
  }
}

static void silk_nlsf2lpc(const silk_nlsf_codebook *_cb,
 opus_int16 _a_q12[],opus_int16 _nlsf_q15[]){
  const unsigned char *ordering;
  opus_int32           c_q17[16];
  opus_int32           p_q16[9];
  opus_int32           q_q16[9];
  opus_int32           a32_q17[16];
  int                  order;
  int                  d_2;
  int                  fi;
  int                  ci;
  order=_cb->order;
  ordering=_cb->ordering;
  for(fi=0;fi<order;fi++){
    int ipart;
    int fpart;
    ipart=_nlsf_q15[fi]>>8;
    fpart=_nlsf_q15[fi]&255;
    c_q17[ordering[fi]]=SILK_COS_Q12[ipart]*(opus_int32)256
     +(SILK_COS_Q12[ipart+1]-SILK_COS_Q12[ipart])*(opus_int32)fpart+4>>3;
  }
  d_2=order>>1;
  silk_nlsf_poly(p_q16,c_q17,d_2);
  silk_nlsf_poly(q_q16,c_q17+1,d_2);
  for(fi=0;fi<d_2;fi++){
    opus_int32 sp_q16;
    opus_int32 dq_q16;
    sp_q16=p_q16[fi]+p_q16[fi+1];
    dq_q16=q_q16[fi]-q_q16[fi+1];
    a32_q17[fi]=dq_q16-sp_q16;
    a32_q17[order-fi-1]=-dq_q16-sp_q16;
  }
  silk_limit_lpc_magnitude(a32_q17,order);
  silk_limit_lpc_gain(a32_q17,order);
  for(ci=0;ci<order;ci++)_a_q12[ci]=(opus_int16)(a32_q17[ci]+16>>5);
}



struct silk_ltp_params{
  opus_int16    subframe_lag[4];
  unsigned char subframe_filter[4];
  int           ltp_scaling_index;
};



static int silk_decode_ltp_params(silk_ltp_params *_ltp_params,
 opus_range_coder *_rc,int _lag,int _coding_type,int _bw,int _nsubframes){
  int delta_lag_index;
  int subframe_cbi;
  int contour_index;
  int lag_min;
  int lag_max;
  int cb_offset;
  int periodicity_index;
  int sfi;
  lag_min=16+8*_bw;
  delta_lag_index=_coding_type==SILK_CODE_CONDITIONALLY?
   orc_dec_pdf(_rc,SILK_PITCH_LAG_DELTA_PDF,8):0;
  if(delta_lag_index==0){
    _lag=orc_dec_pdf(_rc,SILK_PITCH_LAG_HIGH_PDF,8)*(4+2*_bw)
     +orc_dec_pdf(_rc,SILK_PITCH_LAG_LOW_PDFS[_bw],8)+lag_min;
  }
  else _lag+=delta_lag_index-9;
  subframe_cbi=(_bw+1&2)+(_nsubframes>>1)-1;
  contour_index=orc_dec_pdf(_rc,
   SILK_SUBFRAME_PITCH_PDFS+SILK_SUBFRAME_PITCH_PDF_OFFS[subframe_cbi],8);
  lag_max=144+72*_bw;
  cb_offset=SILK_SUBFRAME_PITCH_CB_OFFS[subframe_cbi]
   +contour_index*_nsubframes;
  for(sfi=0;sfi<_nsubframes;sfi++){
    _ltp_params->subframe_lag[sfi]=OPUS_CLAMP(lag_min,
     _lag+SILK_SUBFRAME_PITCH_CBS[cb_offset+sfi],lag_max);
  }
  periodicity_index=orc_dec_pdf(_rc,SILK_PERIODICITY_INDEX_PDF,8);
  cb_offset=SILK_LTP_FILTER_PDF_OFFS[periodicity_index];
  for(sfi=0;sfi<_nsubframes;sfi++){
    _ltp_params->subframe_filter[sfi]=cb_offset
     +orc_dec_pdf(_rc,SILK_LTP_FILTER_PDFS+cb_offset,8);
  }
  _ltp_params->ltp_scaling_index=_coding_type==SILK_CODE_INDEPENDENTLY?
   orc_dec_pdf(_rc,SILK_LTP_SCALE_PDF,8):0;
  return _lag;
}

static void silk_decode_split(opus_int16 _excitation_q15[],
 opus_range_coder *_rc,int _level,int _npulses){
  if(_level<=0)_excitation_q15[0]=_npulses;
  else if(_npulses==0){
    int ei;
    for(ei=0;ei<1<<_level;ei++)_excitation_q15[ei]=0;
  }
  else{
    int split;
    _level--;
    split=orc_dec_pdf(_rc,
     SILK_SPLIT_PDFS+152*_level+(_npulses*(_npulses+1)>>1)-1,8);
    silk_decode_split(_excitation_q15,_rc,_level,split);
    silk_decode_split(_excitation_q15+(1<<_level),_rc,_level,_npulses-split);
  }
}

static void silk_decode_excitation(opus_int16 _excitation_q15[],
 opus_range_coder *_rc,int _frame_type,int _nshell_blocks){
  int rate_level;
  int npulses[20];
  int nlsbs[20];
  int sbi;
  rate_level=orc_dec_pdf(_rc,SILK_RATE_LEVEL_PDFS+9*(_frame_type>>2&1),8);
  for(sbi=0;sbi<_nshell_blocks;sbi++){
    const unsigned char *pdf;
    pdf=SILK_PULSE_COUNT_PDFS+rate_level*18;
    for(nlsbs[sbi]=0;;nlsbs[sbi]++){
      npulses[sbi]=orc_dec_pdf(_rc,pdf,8);
      if(npulses[sbi]<17)break;
      pdf=SILK_PULSE_COUNT_PDFS+(9+(nlsbs[sbi]==9))*18;
    }
  }
  for(sbi=0;sbi<_nshell_blocks;sbi++){
    silk_decode_split(_excitation_q15+16*sbi,_rc,4,npulses[sbi]);
  }
  for(sbi=0;sbi<_nshell_blocks;sbi++){
    int nlsbs_sbi;
    nlsbs_sbi=nlsbs[sbi];
    if(nlsbs_sbi>0){
      int ei;
      for(ei=0;ei<16;ei++){
        int e;
        int lsbi;
        e=_excitation_q15[16*sbi+ei];
        for(lsbi=0;lsbi<nlsbs_sbi;lsbi++)e=2*e+orc_dec_pdf(_rc,SILK_LSB_PDF,8);
        _excitation_q15[16*sbi+ei]=(opus_int16)e;
      }
    }
  }
  for(sbi=0;sbi<_nshell_blocks;sbi++){
    if(npulses[sbi]>0||nlsbs[sbi]>0){
      const unsigned char *pdf;
      int ei;
      pdf=SILK_SIGN_PDFS+14*_frame_type+2*OPUS_MIN(npulses[sbi],6);
      for(ei=0;ei<16;ei++){
        int e;
        e=_excitation_q15[16*sbi+ei];
        if(e){
          int mask;
          mask=orc_dec_pdf(_rc,pdf,8)-1;
          _excitation_q15[16*sbi+ei]=(opus_int16)(e+mask^mask);
        }
      }
    }
  }
}

static opus_int32 silk_inv_gain_q31(opus_int32 _gain_q16){
  opus_int32 gain_qb2;
  opus_int32 inv_gain_qb1p16;
  int        b1;
  int        b2;
  int        inv_qb1;
  int        err_q29;
  b1=silk_ilog(_gain_q16)-2;
  b2=45-b1;
  gain_qb2=_gain_q16<<b2-16;
  inv_qb1=(int)(0x1FFFFFFF/(int)(gain_qb2>>16));
  err_q29=(int)(0x20000000-(opus_int32)(inv_qb1*(opus_int64)gain_qb2>>16));
  inv_gain_qb1p16=(inv_qb1<<16)+(err_q29*inv_qb1>>13);
  return b1<15?
   OPUS_MIN(inv_gain_qb1p16,0x7FFFFFFF>>15-b1)<<15-b1:inv_gain_qb1p16>>b1-15;
}

static opus_int32 silk_gain_adj_q16(opus_int32 _gain_q16,
 opus_int32 _prev_gain_q16){
  opus_int32 prev_gain_qb3;
  opus_int32 gain_qb4;
  opus_int32 gain_adj_qb5;
  opus_int32 err_qb3;
  int        b1;
  int        b2;
  int        b3;
  int        b4;
  int        b5;
  int        inv_qb2;
  if(_gain_q16==_prev_gain_q16)return 65536;
  b1=silk_ilog(_prev_gain_q16)-2;
  b2=silk_ilog(_gain_q16)-2;
  b3=45-b1;
  b4=45-b2;
  b5=29+b2-b1;
  prev_gain_qb3=_prev_gain_q16<<b3-16;
  gain_qb4=_gain_q16<<b4-16;
  inv_qb2=(int)(0x1FFFFFFF/(int)(gain_qb4>>16));
  gain_adj_qb5=(opus_int32)(inv_qb2*(opus_int64)prev_gain_qb3>>16);
  err_qb3=((prev_gain_qb3>>3)-
   (opus_int32)(gain_qb4*(opus_int64)gain_adj_qb5>>32)<<3)
   +(prev_gain_qb3&7);
  gain_adj_qb5+=(opus_int32)(inv_qb2*(opus_int64)err_qb3>>16);
  return b5<16?gain_adj_qb5<<16-b5:gain_adj_qb5>>b5-16;
}

static int silk_decode_frame(silk_channel_ctx *_chan,
 silk_cond_coding_ctx *_cond,opus_range_coder *_rc,
 float *_x,int _active,int _coding_type,int _bw,
 int _nsubframes,int _subframe_size){
  const silk_nlsf_codebook *cb;
  int                       frame_type;
  int                       signal_type;
  int                       quant_offset_type;
  opus_int32                gain_q16[4];
  opus_int16                nlsf_q15[16];
  int                       nlsf_interp_weight_q2;
  silk_ltp_params           ltp_params;
  opus_uint32               seed;
  opus_int16               *excitation_q15;
  int                       frame_size;
  int                       nshell_blocks;
  int                       order;
  frame_type=orc_dec_pdf(_rc,SILK_FRAME_TYPE_PDFS+2*_active,8);
  quant_offset_type=frame_type&1;
  signal_type=frame_type>>1;
  _chan->previous_log_gain=(unsigned char)silk_decode_gains(_rc,gain_q16,
   _chan->previous_log_gain,signal_type,_coding_type,_nsubframes);
  cb=SILK_NLSF_CODEBOOKS[_bw];
  silk_decode_nlsfs(cb,_rc,_x!=NULL?nlsf_q15:NULL,signal_type);
  nlsf_interp_weight_q2=_nsubframes>2?
   orc_dec_pdf(_rc,SILK_NLSF_INTERP_WEIGHT_PDF,8):4;
  if(signal_type==SILK_TYPE_VOICED){
    _cond->previous_lag=(opus_int16)silk_decode_ltp_params(&ltp_params,
     _rc,_cond->previous_lag,_coding_type,_bw,_nsubframes);
  }
  seed=orc_dec_pdf(_rc,SILK_LCG_SEED_PDF,8);
  frame_size=_subframe_size*_nsubframes;
  nshell_blocks=frame_size+15>>4;
  excitation_q15=(opus_int16 *)alloca(nshell_blocks*16*sizeof(*excitation_q15));
  silk_decode_excitation(excitation_q15,_rc,frame_type,nshell_blocks);
  order=cb->order;
  if(_x!=NULL){
    opus_int16  a_q12[16];
    opus_int32  lpc_q29[16];
    opus_int32 *res_q29;
    opus_int32 *ltp_q30;
    opus_int16 *out_q15;
    int         ltp_hist_sz;
    int         offset_q29;
    int         sfi;
    int         ci;
    int         xi;
    ltp_hist_sz=146+72*_bw;
    res_q29=(opus_int32 *)alloca(_subframe_size*sizeof(*res_q29));
    ltp_q30=signal_type==SILK_TYPE_VOICED?
     (opus_int32 *)alloca((ltp_hist_sz+frame_size)*sizeof(*ltp_q30)):NULL;
    out_q15=(opus_int16 *)alloca(_subframe_size*sizeof(*out_q15));
    if(!_chan->nlsf_interpolation_enabled)nlsf_interp_weight_q2=4;
    if(nlsf_interp_weight_q2<4){
      opus_int16 nlsf1_q15[16];
      int        fi;
      for(fi=0;fi<order;fi++){
        nlsf1_q15[fi]=_chan->previous_nlsf_q15[fi]
         +(nlsf_interp_weight_q2*(nlsf_q15[fi]
         -_chan->previous_nlsf_q15[fi])>>2);
      }
      silk_nlsf2lpc(cb,a_q12,nlsf1_q15);
    }
    else silk_nlsf2lpc(cb,a_q12,nlsf_q15);
    offset_q29=SILK_QUANT_OFFSET_Q29[frame_type];
    memcpy(lpc_q29+16-order,_chan->lpc_q29,order*sizeof(*lpc_q29));
    for(sfi=xi=0;sfi<_nsubframes;sfi++){
      opus_int32 gain_q10;
      opus_int32 gain_adj_q16;
      int        ri;
      if(sfi==2&&nlsf_interp_weight_q2<4)silk_nlsf2lpc(cb,a_q12,nlsf_q15);
      for(ri=0;ri<_subframe_size;ri++){
        opus_int32 e_q29;
        opus_int16 e_q15;
        opus_int32 sign_mask;
        e_q15=excitation_q15[xi+ri];
        e_q29=((opus_int32)e_q15<<14)-OPUS_SIGN(e_q15)*1280+offset_q29;
        seed=196314165*seed+907633515;
        sign_mask=-(seed>=0x80000000);
        seed+=e_q15;
        res_q29[ri]=e_q29+sign_mask^sign_mask;
      }
      gain_adj_q16=silk_gain_adj_q16(gain_q16[sfi],_chan->prev_gain_q16);
      _chan->prev_gain_q16=gain_q16[sfi];
      if(signal_type==SILK_TYPE_VOICED){
        opus_int16 b_q14[5];
        int        lag;
        int        li;
        lag=ltp_params.subframe_lag[sfi];
        if(sfi==0||sfi==2&&nlsf_interp_weight_q2<4){
          opus_int32 inv_gain_q31;
          inv_gain_q31=silk_inv_gain_q31(gain_q16[sfi]);
          if(sfi==0){
            inv_gain_q31=(opus_int32)(
             SILK_LTP_SCALE_CB_Q14[ltp_params.ltp_scaling_index]*
             (opus_int64)inv_gain_q31>>16)<<2;
          }
          for(li=ltp_hist_sz-lag-2;li<ltp_hist_sz;li++){
            opus_int32 r_q27;
            r_q27=(opus_int32)_chan->out_q15[li+order]<<12;
            for(ci=0;ci<order;ci++){
              r_q27-=_chan->out_q15[li+order-ci-1]*(opus_int32)a_q12[ci];
            }
            ltp_q30[xi+li]=(opus_int32)(
             OPUS_CLAMP(-32768,r_q27+2048>>12,32767)*
             (opus_int64)inv_gain_q31>>16);
          }
        }
        else if(gain_adj_q16!=65536){
          for(li=ltp_hist_sz-lag-2;li<ltp_hist_sz;li++){
            ltp_q30[xi+li]=
             (opus_int32)(gain_adj_q16*(opus_int64)ltp_q30[xi+li]>>16);
          }
        }
        for(ci=0;ci<5;ci++){
          b_q14[ci]=
           SILK_LTP_FILTER_CBS_Q7[5*ltp_params.subframe_filter[sfi]+ci]<<7;
        }
        for(ri=0;ri<_subframe_size;ri++){
          opus_int32 r_q28;
          r_q28=2;
          for(ci=0;ci<5;ci++){
            r_q28+=(opus_int32)(b_q14[ci]*
             (opus_int64)ltp_q30[ltp_hist_sz+xi-lag+ri+2-ci]>>16);
          }
          res_q29[ri]+=2*r_q28;
          ltp_q30[ltp_hist_sz+xi+ri]=2*res_q29[ri];
        }
      }
      if(gain_adj_q16!=65536){
        for(ci=0;ci<order;ci++){
          lpc_q29[xi-order+ci&15]=
           (opus_int32)(gain_adj_q16*(opus_int64)lpc_q29[xi-order+ci&15]>>16);
        }
      }
      gain_q10=gain_q16[sfi]>>6;
      for(ri=0;ri<_subframe_size;ri++,xi++){
        opus_int32 l_q25;
        l_q25=order>>1;
        for(ci=0;ci<order;ci++){
          l_q25+=(a_q12[ci]*(opus_int64)lpc_q29[xi-ci-1&15]>>16);
        }
        lpc_q29[xi&15]=res_q29[ri]+16*l_q25;
        out_q15[ri]=(opus_int16)OPUS_CLAMP(-32768,
         (opus_int32)(lpc_q29[xi&15]*(opus_int64)gain_q10>>16)+128>>8,32767);
        _x[xi]=out_q15[ri]*(1.0F/32768);
      }
      memmove(_chan->out_q15,_chan->out_q15+_subframe_size,
       (ltp_hist_sz+order-_subframe_size)*sizeof(*_chan->out_q15));
      memcpy(_chan->out_q15+ltp_hist_sz+order-_subframe_size,out_q15,
       _subframe_size*sizeof(*out_q15));
    }
    memcpy(_chan->previous_nlsf_q15,nlsf_q15,order*sizeof(*nlsf_q15));
    _chan->nlsf_interpolation_enabled=1;
    for(ci=0;ci<order;ci++)_chan->lpc_q29[ci]=lpc_q29[xi-order+ci&15];
  }
  return 0;
}

static void silk_stereo_unmix(float *_pcm,
 opus_int16 _prev_stereo_weight_q13[2],opus_int16 _stereo_weight_q13[2],
 float *_mid,float *_side,int _frame_size,int _bw){
  float w00;
  float w01;
  float w10;
  float w11;
  float dw0;
  float dw1;
  float p0;
  int n1;
  int xi;
  n1=64+32*_bw;
  w00=_prev_stereo_weight_q13[0]*(1.0F/8192);
  w01=_prev_stereo_weight_q13[1]*(1.0F/8192);
  w10=_stereo_weight_q13[0]*(1.0F/8192);
  w11=_stereo_weight_q13[1]*(1.0F/8192);
  dw0=(w10-w00)/n1;
  dw1=(w11-w01)/n1;
  for(xi=0;xi<n1;xi++){
    float w0;
    float w1;
    w0=w00+xi*dw0;
    w1=w01+xi*dw1;
    p0=0.25F*(_mid[xi]+2*_mid[xi+1]+_mid[xi+2]);
    _pcm[2*xi+0]=OPUS_CLAMP(-1.0F,(1+w1)*_mid[xi+1]+_side[xi+1]+w0*p0,1.0F);
    _pcm[2*xi+1]=OPUS_CLAMP(-1.0F,(1-w1)*_mid[xi+1]-_side[xi+1]-w0*p0,1.0F);
  }
  for(;xi<_frame_size;xi++){
    p0=0.25F*(_mid[xi]+2*_mid[xi+1]+_mid[xi+2]);
    _pcm[2*xi+0]=OPUS_CLAMP(-1.0F,(1+w11)*_mid[xi+1]+_side[xi+1]+w10*p0,1.0F);
    _pcm[2*xi+1]=OPUS_CLAMP(-1.0F,(1-w11)*_mid[xi+1]-_side[xi+1]-w10*p0,1.0F);
  }
}

static void silk_channel_reset(silk_channel_ctx *_chan){
  memset(_chan,0,sizeof(*_chan));
  _chan->prev_gain_q16=65536;
}

int silk_decode_impl(silk_decoder *_dec,opus_range_coder *_rc,
 float *_pcm,int _nchannels,int _bw,int _fsi){
  silk_cond_coding_ctx  cond[2];
  opus_int16            stereo_weight_q13[2];
  float                *x[2];
  int                   vad[3][2];
  int                   lbrr[3][2];
  int                   stereo_packet;
  int                   stereo_decoder;
  int                   stereo;
  int                   nframes;
  int                   nsubframes;
  int                   subframe_size;
  int                   frame_size;
  int                   prev_mid_only;
  int                   ci;
  int                   fi;
  if(_nchannels<1||_nchannels>2||_bw<SILK_BW_NB||_bw>SILK_BW_WB||
   _fsi<SILK_FS_10MS||_fsi>SILK_FS_60MS){
    return -EINVAL;
  }
  prev_mid_only=_dec->prev_mid_only;
  if(_bw!=_dec->prev_bw){
    silk_channel_reset(_dec->chan+0);
    prev_mid_only=1;
  }
  stereo_packet=_nchannels-1;
  stereo_decoder=_dec->stereo;
  stereo=stereo_packet&&stereo_decoder;
  nframes=_fsi+!_fsi;
  nsubframes=_fsi?4:2;
  subframe_size=40+20*_bw;
  frame_size=subframe_size*nsubframes;
  for(ci=0;ci<=stereo;ci++)x[ci]=(float *)alloca((frame_size+2)*sizeof(*x[ci]));
  silk_decode_header_bits(_rc,vad,lbrr,stereo_packet,nframes);
  /*Skip LBRR.*/
  for(fi=0;fi<nframes;fi++){
    for(ci=0;ci<=stereo_packet;ci++){
      if(lbrr[fi][ci]){
        int coding_type;
        if(ci==0){
          if(stereo_packet)silk_decode_stereo_weights(_rc,NULL);
          if(stereo_packet&&!lbrr[fi][1])orc_dec_pdf(_rc,SILK_MID_ONLY_PDF,8);
        }
        if(fi>0&&lbrr[fi-1][ci])coding_type=SILK_CODE_CONDITIONALLY;
        else coding_type=SILK_CODE_INDEPENDENTLY;
        silk_decode_frame(NULL,cond+ci,_rc,NULL,
         1,coding_type,_bw,nsubframes,subframe_size);
      }
    }
  }
  for(fi=0;fi<nframes;fi++){
    int mid_only;
    if(stereo_packet)silk_decode_stereo_weights(_rc,stereo_weight_q13);
    mid_only=!stereo_packet||!vad[fi][1]&&orc_dec_pdf(_rc,SILK_MID_ONLY_PDF,8);
    if(!mid_only&&prev_mid_only)silk_channel_reset(_dec->chan+1);
    for(ci=0;ci<=!mid_only;ci++){
      int coding_type;
      if(fi==0)coding_type=SILK_CODE_INDEPENDENTLY;
      else if(ci==0||!prev_mid_only)coding_type=SILK_CODE_CONDITIONALLY;
      else coding_type=SILK_CODE_INDEPENDENTLY_NO_LTP_SCALING;
      if(ci<=stereo_decoder){
        silk_decode_frame(_dec->chan+ci,cond+ci,_rc,x[ci]+2,
         vad[fi][ci],coding_type,_bw,nsubframes,subframe_size);
      }
      else{
        silk_decode_frame(NULL,cond+ci,_rc,NULL,
         vad[fi][ci],coding_type,_bw,nsubframes,subframe_size);
      }
    }
    if(ci<=stereo)memset(x[1]+2,0,frame_size*sizeof(*x[1]));
    for(ci=0;ci<=stereo;ci++){
      x[ci][0]=_dec->chan[ci].unmix[0];
      x[ci][1]=_dec->chan[ci].unmix[1];
      _dec->chan[ci].unmix[0]=x[ci][frame_size];
      _dec->chan[ci].unmix[1]=x[ci][frame_size+1];
    }
    if(stereo){
      silk_stereo_unmix(_pcm+2*fi*frame_size,
       _dec->prev_stereo_weight_q13,stereo_weight_q13,x[0],x[1],frame_size,_bw);
    }
    else if(stereo_decoder){
      int xi;
      for(xi=0;xi<frame_size;xi++)_pcm[2*xi+1]=_pcm[2*xi]=x[0][xi+1];
    }
    else{
      memcpy(_pcm+fi*frame_size,x[0]+1,frame_size*sizeof(*_pcm));
      mid_only=1;
    }
    prev_mid_only=mid_only;
  }
  _dec->prev_bw=(unsigned char)_bw;
  _dec->prev_mid_only=(unsigned char)prev_mid_only;
  return frame_size*nframes;
}

int silk_decode(silk_decoder *_dec,float *_pcm,
 const unsigned char *_data,opus_uint32 _len,
 int _nchannels,int _bw,int _fsi){
  opus_range_coder rc;
  orc_dec_init(&rc,_data,_len);
  return silk_decode_impl(_dec,&rc,_pcm,_nchannels,_bw,_fsi);
}
